/**********************************************************************
 * 
 *  Toby Opferman
 *
 *  Example Application which uses shared memory to display the contents
 *  of a virtual second monitor
 *
 *  This example is for educational purposes only.  I license this source
 *  out for use in learning.
 *
 *  Copyright (c) 2005, All Rights Reserved  
 **********************************************************************/
 
 
 #include <windows.h>
 #include <gdi.h>
 #include "virtualmonitor.h"
 #include "videomemory.h"
 
 typedef struct _VIRTUALMONITOR
 {
    HWND hWnd;
    HGDI hGDI;
    PCHAR pVideoMemory;

 } VIRTUALMONITOR, *PVIRTUALMONITOR;
 

 /***********************************************************************
  * VirtualMonitor_Initialize
  *  
  *    Virtual Monitor Initialize
  *
  * Parameters
  *     Window Handle
  * 
  * Return Value
  *     Handle To Virtual Monitor
  *
  ***********************************************************************/
 HVMON VirtualMonitor_Initialize(HWND hWnd, UINT uiPid, UINT uiOffset)
 {
     PVIRTUALMONITOR pVirtualMonitor = NULL;

     pVirtualMonitor = LocalAlloc(LMEM_ZEROINIT, sizeof(VIRTUALMONITOR));

     if(pVirtualMonitor)
     {
        pVirtualMonitor->hWnd = hWnd;
        pVirtualMonitor->hGDI = GDI_Init(hWnd, 640, 480);
        pVirtualMonitor->pVideoMemory = VideoMemory_GetSharedMemory(7564, 0x27f8000 );
     }

     return (HVMON)pVirtualMonitor;
 }


 /***********************************************************************
  * VirtualMonitor_Update
  *  
  *    Updates the Screen Image from the shared memory with the
  *    video driver
  *
  * Parameters
  *     Handle To Virtual Monitor
  * 
  * Return Value
  *     None
  *
  ***********************************************************************/
 void VirtualMonitor_Update(HVMON hVmon)
 {
     PVIRTUALMONITOR pVirtualMonitor = (PVIRTUALMONITOR)hVmon;
     PCHAR pVirtualMonitorBuffer;

     pVirtualMonitorBuffer = GDI_BeginPaint(pVirtualMonitor->hGDI);

     memcpy(pVirtualMonitorBuffer, pVirtualMonitor->pVideoMemory, 640*480*4); 
     
     GDI_EndPaint(pVirtualMonitor->hGDI);
 }


 /***********************************************************************
  * VirtualMonitor_GetVideoDC
  *  
  *    Updates the Screen Image from the shared memory with the
  *    video driver
  *
  * Parameters
  *     Handle To Virtual Monitor
  * 
  * Return Value
  *     Video HDC
  *
  ***********************************************************************/
 HDC VirtualMonitor_GetVideoDC(HVMON hVmon)
 {
    HDC hDC;
    PVIRTUALMONITOR pVirtualMonitor = (PVIRTUALMONITOR)hVmon;

    hDC = GDI_GetDC(pVirtualMonitor->hGDI);

    return hDC;
 }

 /***********************************************************************
  * VirtualMonitor_ReleaseVideoDC
  *  
  *    Releases the Video DC
  *
  * Parameters
  *     Handle To Virtual Monitor, handle to DC
  * 
  * Return Value
  *     None
  *
  ***********************************************************************/
 void VirtualMonitor_ReleaseVideoDC(HVMON hVmon, HDC hVideoDC)
 {
     PVIRTUALMONITOR pVirtualMonitor = (PVIRTUALMONITOR)hVmon;

     GDI_ReleaseDC(pVirtualMonitor->hGDI);
 }
