/**********************************************************************
 * 
 *  Toby Opferman
 *
 *  VMWare Guest OS - Allocating Memory To Be Shared
 *
 *  This example is for educational purposes only.  I license this source
 *  out for use in learning how to write a device driver.
 *
 *  Copyright (c) 2005, All Rights Reserved  
 **********************************************************************/


#define _X86_

#include <ddk\ntddk.h>
#include "vmware.h"

#pragma alloc_text(PAGE, VMWareGuest_AllocateSharedMemory)
#pragma alloc_text(PAGE, VMWareGuest_FreeSharedMemory)          

/*********************************************************************
 * VMWareGuest_AllocateSharedMemory
 *
 *   This function is used to allocate memory for use by a video
 *   display in a VMWare Guest OS to be used by the Host OS.
 *
 *********************************************************************/
PVOID VMWareGuest_AllocateSharedMemory(PVOID pHwExt, UINT uiSize)
{
   PHYSICAL_ADDRESS PhsyicalAddress = { 0xFFFFFFFF };

   return MmAllocateContiguousMemory((SIZE_T)uiSize, PhsyicalAddress);
}

/*********************************************************************
 * VMWareGuest_SetRegistry
 *
 *   Puts the physical address into the registry so it can be manually
 *   used in the host operating system
 *
 *********************************************************************/
void VMWareGuest_SetRegistry(PVOID pMemory)
{
    PHYSICAL_ADDRESS PhysicalMemory;

    PhysicalMemory = MmGetPhysicalAddress(pMemory);

    RtlCreateRegistryKey(RTL_REGISTRY_USER, L"Software\\Opferman");
    RtlWriteRegistryValue(RTL_REGISTRY_USER, L"Software\\Opferman", L"VMWarePhysicalMemory", REG_DWORD, &PhysicalMemory, 4);
}

/*********************************************************************
 * VMWareGuest_FreeSharedMemory
 *
 *   This function is used to free memory that was allocated for use by
 *   an application in the Host OS.
 *
 *********************************************************************/
void VMWareGuest_FreeSharedMemory(PVOID pHwExt, PVOID pMemory)
{
   MmFreeContiguousMemory(pMemory);
}
